package SPVM::Float;

1;

=head1 NAME

SPVM::Float - Float Utility

=head1 CONSTANT FUNCTIONS

=head2 MIN_VALUE

B<sub MIN_VALUE : float ()>

A constant holding the minimum value a float can have, C<-128>.

=head2 MAX_VALUE

B<sub MAX_VALUE : float ()>

A constant holding the maximum value a float can have, C<127>.

=head2 SIZE

B<sub SIZE : int ()>

The number of bits used to represent a float value in two's complement binary form.

=head2 BYTES

B<sub BYTES : int ()>

The number of floats used to represent a float value in two's complement binary form.

=head1 FUNCTIONS

