/* A Bison parser, made by GNU Bison 2.3.  */

/* Skeleton interface for Bison's Yacc-like parsers in C

   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
   Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.  */

/* As a special exception, you may create a larger work that contains
   part or all of the Bison parser skeleton and distribute that work
   under terms of your choice, so long as that work isn't itself a
   parser generator using the skeleton or a modified version thereof
   as a parser skeleton.  Alternatively, if you modify or redistribute
   the parser skeleton itself, you may (at your option) remove this
   special exception, which will cause the skeleton and the resulting
   Bison output files to be licensed under the GNU General Public
   License without this special exception.

   This special exception was added by the Free Software Foundation in
   version 2.2 of Bison.  */

/* Tokens.  */
#ifndef YYTOKENTYPE
# define YYTOKENTYPE
   /* Put the tokens into the symbol table, so that GDB and other debuggers
      know about them.  */
   enum yytokentype {
     MY = 258,
     HAS = 259,
     SUB = 260,
     PACKAGE = 261,
     IF = 262,
     ELSIF = 263,
     ELSE = 264,
     RETURN = 265,
     FOR = 266,
     WHILE = 267,
     USE = 268,
     NEW = 269,
     SET = 270,
     GET = 271,
     OUR = 272,
     LAST = 273,
     NEXT = 274,
     NAME = 275,
     CONSTANT = 276,
     ENUM = 277,
     DESCRIPTOR = 278,
     CORETYPE = 279,
     UNDEF = 280,
     CROAK = 281,
     VAR_NAME = 282,
     SWITCH = 283,
     CASE = 284,
     DEFAULT = 285,
     VOID = 286,
     EVAL = 287,
     BYTE = 288,
     SHORT = 289,
     INT = 290,
     LONG = 291,
     FLOAT = 292,
     DOUBLE = 293,
     STRING = 294,
     WEAKEN = 295,
     SPECIAL_ASSIGN = 296,
     ASSIGN = 297,
     OR = 298,
     AND = 299,
     BIT_XOR = 300,
     BIT_OR = 301,
     BIT_AND = 302,
     REL = 303,
     SHIFT = 304,
     REMAINDER = 305,
     DIVIDE = 306,
     MULTIPLY = 307,
     UMINUS = 308,
     ARRAY_LENGTH = 309,
     NOT = 310,
     DEC = 311,
     INC = 312,
     ARROW = 313
   };
#endif
/* Tokens.  */
#define MY 258
#define HAS 259
#define SUB 260
#define PACKAGE 261
#define IF 262
#define ELSIF 263
#define ELSE 264
#define RETURN 265
#define FOR 266
#define WHILE 267
#define USE 268
#define NEW 269
#define SET 270
#define GET 271
#define OUR 272
#define LAST 273
#define NEXT 274
#define NAME 275
#define CONSTANT 276
#define ENUM 277
#define DESCRIPTOR 278
#define CORETYPE 279
#define UNDEF 280
#define CROAK 281
#define VAR_NAME 282
#define SWITCH 283
#define CASE 284
#define DEFAULT 285
#define VOID 286
#define EVAL 287
#define BYTE 288
#define SHORT 289
#define INT 290
#define LONG 291
#define FLOAT 292
#define DOUBLE 293
#define STRING 294
#define WEAKEN 295
#define SPECIAL_ASSIGN 296
#define ASSIGN 297
#define OR 298
#define AND 299
#define BIT_XOR 300
#define BIT_OR 301
#define BIT_AND 302
#define REL 303
#define SHIFT 304
#define REMAINDER 305
#define DIVIDE 306
#define MULTIPLY 307
#define UMINUS 308
#define ARRAY_LENGTH 309
#define NOT 310
#define DEC 311
#define INC 312
#define ARROW 313




#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
typedef int YYSTYPE;
# define yystype YYSTYPE /* obsolescent; will be withdrawn */
# define YYSTYPE_IS_DECLARED 1
# define YYSTYPE_IS_TRIVIAL 1
#endif



